package com.kingdee.eas.custom.task;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.gemdale.gdgateway.util.MD5Utils;
import com.kingdee.eas.base.permission.UserCollection;
import com.kingdee.eas.base.permission.UserFactory;
import com.kingdee.eas.base.permission.UserInfo;
import com.kingdee.eas.basedata.person.PersonInfo;
import com.kingdee.eas.custom.task.vo.EmpInfo;
import com.kingdee.eas.util.app.DbUtil;
import com.kingdee.util.DateTimeUtils;
import com.kingdee.util.StringUtils;
import okhttp3.*;
import org.apache.log4j.Logger;

import java.io.FileInputStream;
import java.io.IOException;

import com.kingdee.bos.*;

import java.lang.String;
import java.text.SimpleDateFormat;
import java.util.*;

public class SyncZYEmpFacadeControllerBean extends AbstractSyncZYEmpFacadeControllerBean {
    private static String propertiesUrl = System.getProperty("EAS_HOME") + "/server/properties/Api/propertie.properties";
    private List<EmpInfo> empInfoList = new ArrayList<>();
    private static Logger logger = Logger.getLogger(SyncZYEmpFacadeControllerBean.class);
    private int pageSize = 100;

    /**
     * ͬû
     *
     * @param ctx
     * @throws BOSException
     */
    @Override
    protected void _SyncAllEmp(Context ctx) throws BOSException {
        super._SyncAllEmp(ctx);
        Properties properties = new Properties();
        try {
            properties.load(new FileInputStream(propertiesUrl));
        } catch (IOException e) {
            e.printStackTrace();
            throw new BOSException(String.format("ȡʧܣ[%s]", propertiesUrl));
        }
        int empCount = getEmpCount(properties, null, null);
        logger.error("ȡû: " + empCount);
        getEmp(properties, null, null, empCount);
        //ûϢ
        updateUserInfo(ctx);
    }

    /**
     * ûϢ
     *
     * @param ctx
     * @throws BOSException
     */
    private void updateUserInfo(Context ctx) throws BOSException {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            UserCollection userCollection = UserFactory.getLocalInstance(ctx).getUserCollection("select id,person.*");
            Map<String, String> idCardNoMap = new HashMap<>();
            Map<String, String> passportNoMap = new HashMap<>();
            Map<String, String> emailMap = new HashMap<>();
            for (int i = 0; i < userCollection.size(); i++) {
                UserInfo userInfo = userCollection.get(i);
                String userId = userInfo.getId().toString();
                PersonInfo personInfo = userInfo.getPerson();
                //֤
                String idCardNO = personInfo.getIdCardNO();
                if (!StringUtils.isEmpty(idCardNO)) {
                    idCardNoMap.put(idCardNO, userId);
                }
                //պ
                String passportNO = personInfo.getPassportNO();
                if (!StringUtils.isEmpty(passportNO)) {
                    passportNoMap.put(passportNO, userId);
                }
                //
                String email = personInfo.getEmail();
                if (!StringUtils.isEmpty(email)) {
                    emailMap.put(email, userId);
                }
            }
            List<Object[]> updateParams = new ArrayList<>();
            for (int i = 0; i < empInfoList.size(); i++) {
                //ƥ֤պ,ƥ
                EmpInfo empInfo = empInfoList.get(i);
                //˺
                String username = empInfo.getUsername();
                String email = empInfo.getEmail();
                String cardType = empInfo.getCardType();
                String cardNum = empInfo.getCardNum();
                String userId = null;
                if (!StringUtils.isEmpty(cardNum)) {
                    if ("½֤".equals(cardType)) {
                        userId = idCardNoMap.get(cardNum);
                    } else {
                        userId = passportNoMap.get(cardNum);
                    }
                } else {
                    userId = emailMap.get(email);
                }
                if (!StringUtils.isEmpty(userId)) {
                    updateParams.add(new Object[]{username, userId});
                }
            }
            logger.error("ûϢ: " + objectMapper.writeValueAsString(updateParams));
            String updateSql = "update t_pm_user set CFZYLoginName=? where fid=?";
            DbUtil.executeBatch(ctx, updateSql, updateParams);
        } catch (Exception e) {
            e.printStackTrace();
            throw new BOSException("ûϢ: " + e.getMessage());
        }
    }

    /**
     * ݸʱͬû
     *
     * @param ctx
     * @param updateBeginTime ¿ʼʱ䣨yyyy-MM-dd HH:mm:ss
     * @param updateEndTime   ½ʱ䣨yyyy-MM-dd HH:mm:ss
     * @throws BOSException
     */
    @Override
    protected void _SyncEmpByUpdateTime(Context ctx, String updateBeginTime, String updateEndTime) throws BOSException {
        super._SyncEmpByUpdateTime(ctx, updateBeginTime, updateEndTime);
        Properties properties = new Properties();
        try {
            properties.load(new FileInputStream(propertiesUrl));
        } catch (IOException e) {
            e.printStackTrace();
            throw new BOSException(String.format("ȡʧܣ[%s]", propertiesUrl));
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        if (StringUtils.isEmpty(updateBeginTime) && StringUtils.isEmpty(updateEndTime)) {
            updateBeginTime = sdf.format(DateTimeUtils.addDay(new Date(), -7));
        }
        int empCount = getEmpCount(properties, updateBeginTime, updateEndTime);
        logger.error("ȡû: " + empCount);
        getEmp(properties, updateBeginTime, updateEndTime, empCount);
        //ûϢ
        updateUserInfo(ctx);
    }

    /**
     * ȡmd5ǩ
     *
     * @param appSecret
     * @param path
     * @return
     * @throws BOSException
     */
    private String getSignMd5String(String appSecret, String path) throws BOSException {
        StringBuilder errorMsg = new StringBuilder();
        if (StringUtils.isEmpty(appSecret)) {
            errorMsg.append("appSecretΪ!").append("\n");
        }
        if (StringUtils.isEmpty(path)) {
            errorMsg.append("pathΪ!").append("\n");
        }
        if (errorMsg.length() > 0) {
            throw new BOSException("md5ǩ: " + errorMsg);
        }
        String sign = MD5Utils.sign(appSecret, path);
        logger.error("md5ǩ: " + sign);
        return sign;
    }

    /**
     * ѯû
     *
     * @param properties
     * @param updateBeginTime ¿ʼʱ
     * @param updateEndTime   ½ʱ
     * @return
     * @throws BOSException
     */
    private int getEmpCount(Properties properties, String updateBeginTime, String updateEndTime) throws BOSException {
        StringBuilder errorMsg = new StringBuilder();
        OkHttpClient client = new OkHttpClient();
        String appkey = properties.getProperty("Appkey");
        String appSecret = properties.getProperty("AppSecret");
        //"https://api-test.gemdale.com/openApi/organization-center/emp/getEmpCount"
        String serverName = properties.getProperty("serverName");
        //ȡûӿڵַ
        String getEmpCountPath = properties.getProperty("getEmpCountPath");
        if (StringUtils.isEmpty(appkey)) {
            errorMsg.append("appkey Ϊ!").append("\n");
        }
        if (StringUtils.isEmpty(appSecret)) {
            errorMsg.append("appSecret Ϊ!").append("\n");
        }
        if (StringUtils.isEmpty(serverName)) {
            errorMsg.append("serverName Ϊ!").append("\n");
        }
        if (StringUtils.isEmpty(getEmpCountPath)) {
            errorMsg.append("ȡûӿڵַ Ϊ!").append("\n");
        }
        if (errorMsg.length() > 0) {
            throw new BOSException("md5ǩ: " + errorMsg);
        }
        JSONObject content = new JSONObject();
        content.put("containslnvalid", false);//ǷʧЧû
        if (!StringUtils.isEmpty(updateBeginTime)) {
            content.put("updateBeginTime", updateBeginTime);
        }
        if (!StringUtils.isEmpty(updateEndTime)) {
            content.put("updateEndTime", updateEndTime);
        }
        String contentStr = content.toJSONString();
        logger.error("ѯû: " + contentStr);
        RequestBody body = RequestBody.create(MediaType.parse("application/json"), contentStr);
        try {
            Request request = new Request.Builder()
                    .url(serverName + getEmpCountPath)
                    .post(body)
                    .addHeader("x-ca-key", appkey)
                    .addHeader("x-ca-signature", getSignMd5String(appSecret, getEmpCountPath))
                    .addHeader("x-ca-signature-method", "MD5")
                    .addHeader("content-type", "application/json")
                    .build();
            Response response = client.newCall(request).execute();
            if (response.isSuccessful()) {
                ObjectMapper objectMapper = new ObjectMapper();
                Map result = objectMapper.readValue(response.body().string(), Map.class);
                Boolean success = (Boolean) result.get("success");
                if (success) {
                    return (int) result.get("total");
                } else {
                    String msg = (String) result.get("msg");
                    throw new BOSException(msg);
                }
            } else {
                throw new BOSException("ʧ");
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new BOSException("ѯû: " + e.getMessage());
        }
    }

    /**
     * ѯû
     *
     * @param properties
     * @param updateBeginTime ¿ʼʱ
     * @param updateEndTime   ½ʱ
     * @return
     * @throws BOSException
     */
    private void getEmp(Properties properties, String updateBeginTime, String updateEndTime, int empCount) throws BOSException {
        if (empCount > 0) {
            ObjectMapper objectMapper = new ObjectMapper();
            StringBuilder errorMsg = new StringBuilder();
            OkHttpClient client = new OkHttpClient();
            String appkey = properties.getProperty("Appkey");
            String appSecret = properties.getProperty("AppSecret");
            String serverName = properties.getProperty("serverName");
            //ȡûӿڵַ
            String getEmpPath = properties.getProperty("getEmpPath");
            if (StringUtils.isEmpty(appkey)) {
                errorMsg.append("appkey Ϊ!").append("\n");
            }
            if (StringUtils.isEmpty(appSecret)) {
                errorMsg.append("appSecret Ϊ!").append("\n");
            }
            if (StringUtils.isEmpty(serverName)) {
                errorMsg.append("serverName Ϊ!").append("\n");
            }
            if (StringUtils.isEmpty(getEmpPath)) {
                errorMsg.append("ȡûӿڵַ Ϊ!").append("\n");
            }
            if (errorMsg.length() > 0) {
                throw new BOSException("md5ǩ: " + errorMsg);
            }
            String signMd5String = getSignMd5String(appSecret, getEmpPath);
            int pageCount = empCount % pageSize == 0 ? empCount / pageSize : empCount / pageSize + 1;
            for (int i = 1; i <= pageCount; i++) {
                JSONObject content = new JSONObject();
                content.put("containslnvalid", false);//ǷʧЧû
                content.put("pageSize", pageSize);
                content.put("pageNum", i);
                if (!StringUtils.isEmpty(updateBeginTime)) {
                    content.put("updateBeginTime", updateBeginTime);
                }
                if (!StringUtils.isEmpty(updateEndTime)) {
                    content.put("updateEndTime", updateEndTime);
                }
                RequestBody body = RequestBody.create(MediaType.parse("application/json"), content.toJSONString());
                try {
                    Request request = new Request.Builder()
                            .url(serverName + getEmpPath)
                            .post(body)
                            .addHeader("x-ca-key", appkey)
                            .addHeader("x-ca-signature", signMd5String)
                            .addHeader("x-ca-signature-method", "MD5")
                            .addHeader("content-type", "application/json")
                            .build();
                    Response response = client.newCall(request).execute();
                    if (response.isSuccessful()) {
                        String resultStr = response.body().string();
                        Map result = objectMapper.readValue(resultStr, Map.class);
                        Boolean success = (Boolean) result.get("success");
                        logger.error("ѯû: " + resultStr);
                        if (success) {
                            String data = (String) result.get("data");
                            EmpInfo empInfo = objectMapper.readValue(data, EmpInfo.class);
                            empInfoList.add(empInfo);
                        } else {
                            String msg = (String) result.get("msg");
                            throw new BOSException(msg);
                        }
                    } else {
                        throw new BOSException("ʧ");
                    }
                    Thread.sleep(100);
                } catch (Exception e) {
                    e.printStackTrace();
                    throw new BOSException("ѯû: " + e.getMessage());
                }
            }
        }
    }
}